#include "MTI/PluginInterface.h"
#include "MTI/PluginFactory.h"
#include "MTI/PluginInstance.h"
#include "MTI/ModelTraceInterface.h"
#include "MTI/ParameterInterface.h"
#include "eslapi/CAInterfaceRegistry.h"
#include "sg/SGComponentRegistry.h"

#include <list>
#include <string>
#include <algorithm>
#include <cstdio>
#include <sys/stat.h>

using namespace std;
using namespace eslapi;
using namespace MTI;

#define PLUGIN_VERSION "0.0.1"

#define ITM_PORTS   32

/////////////////////////////////////////////////////////////////////////////

class ITMtrace :
    public MTI::PluginInstance,
    public MTI::ParameterInterface
{
public:
    virtual eslapi::CAInterface * ObtainInterface(eslapi::if_name_t    ifName,
                                                  eslapi::if_rev_t     minRev,
                                                  eslapi::if_rev_t *   actualRev);

    ITMtrace(const char *instance_name,
                       uint32_t num_parameters,
                       eslapi::CADIParameterValue_t *parameter_values);

    ~ITMtrace();


    /** This is to associate a plugin with a simulation instance. Exactly one simulation must be registered. */
    virtual eslapi::CADIReturn_t RegisterSimulation(eslapi::CAInterface *simulation);

    // This is called before the plugin .dll/.so is unloaded and should allow the plugin to do it's cleanup.
    virtual void Release();

    virtual const char *GetName() const;

    virtual CADIReturn_t GetParameterInfos(uint32_t startIndex,
                                           uint32_t desiredNumOfParams,
                                           uint32_t *actualNumOfParams,
                                           CADIParameterInfo_t *params);

    virtual CADIReturn_t GetParameterInfo(const char *parameterName,
                                          CADIParameterInfo_t *param);

    virtual CADIReturn_t GetParameterValues(uint32_t parameterCount,
                                            uint32_t *actualNumOfParamsRead,
                                            CADIParameterValue_t *paramValuesOut);

    virtual CADIReturn_t SetParameterValues(uint32_t parameterCount,
                                            CADIParameterValue_t *parameters,
                                            CADIFactoryErrorMessage_t *error);

public: // methods
    MTI::EventClass *event_class;

    static void
    TraceITM_Thunk(void * user_data,
                  const MTI::EventClass *event_class,
                  const struct MTI::EventRecord *record);

    eslapi::CADIReturn_t Error(const char *message) const;

private:
    // Interface registry for a convenient implementation of ObtainInterface().
    eslapi::CAInterfaceRegistry interfaceRegistry;
    std::string instance_name;
    MTI::ValueIndex itm_payload_index;
    MTI::ValueIndex itm_hdr_index;

    string trace_file_name;
    FILE *trace_file[ITM_PORTS]; // file for each ITM channel

    void DeleteEmptyFile(const char *filename);
    void CloseTraceFile();
    void SetTraceFile(const char *new_filename);

    void
    TraceITM(const MTI::EventClass *event_class,
            const struct MTI::EventRecord *record);

    CADIReturn_t SetParametersInternal(uint32_t num_parameters,
                                       CADIParameterValue_t *parameters);

};

class ThePluginFactory :
    public MTI::PluginFactory
{
public:
    ThePluginFactory() : interfaceRegistry("ThePluginFactory")
    {
        interfaceRegistry.Register<MTI::PluginFactory>(this);
    }

    virtual eslapi::CAInterface * ObtainInterface(eslapi::if_name_t    ifName,
                                                  eslapi::if_rev_t     minRev,
                                                  eslapi::if_rev_t *   actualRev);

    virtual uint32_t GetNumberOfParameters();

    virtual eslapi::CADIReturn_t GetParameterInfos(eslapi::CADIParameterInfo_t *parameter_info_list);

    virtual eslapi::CAInterface *Instantiate(const char *instance_name,
                                             uint32_t number_of_parameters,
                                             eslapi::CADIParameterValue_t *parameter_values);

    virtual void Release();

    virtual const char *GetType() const { return "ITMtrace"; }
    virtual const char *GetVersion() const { return PLUGIN_VERSION; }

private:
    // Interface registry for a convenient implementation of ObtainInterface().
    eslapi::CAInterfaceRegistry interfaceRegistry;
};
