//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2004-2008 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//-----------------------------------------------------------------------------

/*-----------------------------------------------------------------------------
// Copyright (c) 2010-2011 by Carbon Design Systems, Inc., All Rights Reserved.
//
// THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF CARBON
// DESIGN SYSTEMS, INC.  PERMISSION IS HEREBY GRANTED TO USE, MODIFY, AND/OR COPY
// THIS SOFTWARE FOR INTERNAL USE ONLY PROVIDED THAT THE ABOVE COPYRIGHT NOTICE
// APPEARS IN ALL COPIES OF THIS SOFTWARE.
//
//---------------------------------------------------------------------------*/

#include <stdio.h>
#include <rt_misc.h>

// Heap base from scatter file
extern int Image$$ARM_LIB_STACKHEAP$$ZI$$Base;


/////////////////////////////////////////////////////////////////////////////
//
// This function re-implements the C Library semihosted function. The stack
// pointer has aready been set and is passed back to the funtion, The base of
// the heap is set from the scatter file.
//
/////////////////////////////////////////////////////////////////////////////

__value_in_regs struct __initial_stackheap __user_initial_stackheap(
        unsigned R0, unsigned SP, unsigned R2, unsigned SL)
{
    struct __initial_stackheap config;

    config.heap_base = (unsigned int)&Image$$ARM_LIB_STACKHEAP$$ZI$$Base; // placed by scatterfile
    config.stack_base = SP;   // inherit SP from the execution environment

    return config;
}

#include <stdio.h>
#include <time.h>
#include <stdint.h>

#define TRUE 1
#define FALSE 0

/*
** Importing __use_no_semihosting ensures that our image doesn't link
** with any C Library code that makes direct use of semihosting. 
**
*/

#ifdef ITM
#define USE_ITM
#pragma import(__use_no_semihosting)
#endif
  

/*
** Retargeted I/O
** ==============
** The following C library functions make use of semihosting
** to read or write characters to the debugger console: fputc(),
** fgetc(), and _ttywrch().  They must be retargeted to write to
** the Integrator AP UART.  __backspace() must also be retargeted
** with this layer to enable scanf().  See the Compiler and
** Libraries Guide.
*/

/*
** These must be defined to avoid linking in stdio.o from the
** C Library
*/
#ifdef ITM
struct __FILE { int handle;   /* Add whatever you need here */};
#endif
FILE __stdout;
FILE __stdin;
FILE __stderr;

#define ITM_Port8(n)    (*((volatile unsigned char *)(0xe0000000+4*n)))
#define ITM_Port16(n)   (*((volatile unsigned short*)(0xe0000000+4*n)))
#define ITM_Port32(n)   (*((volatile unsigned long *)(0xe0000000+4*n)))

#define ITM_TER   (*((unsigned long *) 0xe0000e00))
#define ITM_TCR   (*((unsigned long *) 0xe0000e80))

#define DEMCR           (*((volatile unsigned long *)(0xe000EDFC)))
#define TRCENA          0x01000000

#define 	ITM_RXBUFFER_EMPTY   0x5AA55AA5

#ifdef USE_ITM

int32_t ITM_RxBuffer = ITM_RXBUFFER_EMPTY;

/*
** __backspace must return the last char read to the stream
** fgetc() needs to keep a record of whether __backspace was
** called directly before it
*/
int last_char_read;
int backspace_called;

int32_t ITM_SendChar (int32_t ch) {
    if ((ITM_TCR & (1UL << 0)) &&         /* ITM enabled */
        (DEMCR & TRCENA) &&
        (ITM_TER & (1UL << 0))) 
    { 
        while (ITM_Port32(0) == 0);
        ITM_Port8(0) = (uint8_t)ch;
    }
    return (ch);
}
 
int fputc(int ch, FILE *f)
{
    (void) ITM_SendChar(ch);

    return(ch);
}

int32_t ITM_ReceiveChar (void) {
    int32_t ch = -1;                      /* no character available */
 
    if (ITM_RxBuffer != ITM_RXBUFFER_EMPTY) 
    {
        ch = ITM_RxBuffer;
        ITM_RxBuffer = ITM_RXBUFFER_EMPTY;  /* ready for next character */
    }
 
    return (ch);
}

int fgetc(FILE *f)
{
    unsigned char tempch;

    /* if we just backspaced, then return the backspaced character */
    /* otherwise output the next character in the stream */
    if (backspace_called == TRUE)
    {
      backspace_called = FALSE;
      return last_char_read;
    }

    tempch = ITM_ReceiveChar();
    last_char_read = (int)tempch;       /* backspace must return this value */

	fputc(tempch, f);
	if (tempch == '\r')
	{
	 fputc('\n', f);
	}

    return tempch;
}

void _ttywrch(int ch)
{
    ITM_Port8(0) = ch;
}

/*
** The effect of __backspace() should be to return the last character
** read from the stream, such that a subsequent fgetc() will
** return the same character again.
*/

int __backspace(FILE *f)
{
    backspace_called = TRUE;
    return 0;
}

/* END of Retargeted I/O */
#endif        //end of '#ifdef USE_ITM'


/*
** Exception Signaling and Handling
** ================================
** The C library implementations of ferror() uses semihosting directly
** and must therefore be retargeted.  This is a minimal reimplementation.  
** _sys_exit() is called after the user's main() function has exited.  The C library
** implementation uses semihosting to report to the debugger that the application has
** finished executing.  
*/


#ifdef ITM

int ferror(FILE *f)
{
    return EOF;
}

void _sys_exit(int return_code)
{
	
}

#endif        //end of '#ifdef ITM

// End of File retarget.c
